using System;
using System.Collections.Generic;
using Action;
using MultiPlayer;
using NetworkBehaviourClasses;
using Unity.Netcode;
using Unity.VisualScripting;
using Unity.VisualScripting.FullSerializer;
using UnityEngine;
using UnityEngine.Tilemaps;

namespace Model
{
    public class ActionValidator
    {
        // private fields
        private GameManager _gameManager;
        private List<InputData> _playerInputsAndActions;
        private List<RobotAction> _actions;
        
        // events
        public static event Action<List<RobotAction>> OnActionsValidated;

        // Constructor
        public ActionValidator( List<InputData>  playerInputsAndActions, GameManager gameManager)
        {
            this._playerInputsAndActions = playerInputsAndActions;
            this._gameManager = gameManager;
            _actions = new List<RobotAction>();
        }

        // public methods
        public void Validate()
        {
            _playerInputsAndActions.Sort();
            
            foreach (var x in _playerInputsAndActions)
            {
                NetworkObject nobj;
                x.PlayerObject.TryGet(out nobj);
                GameObject playerObject = nobj.gameObject;
                
                switch (x.Type)
                {
                    case ActionType.Move:
                        ValidateMove(playerObject, x.Inputs);
                        break;
                    case ActionType.Wait:
                        ValidateWait(playerObject, x.Inputs);
                        break;
                    case ActionType.Clean:
                        ValidateClean(playerObject, x.Inputs);
                        break;
                    case ActionType.Rotate:
                        ValidateRotate(playerObject, x.Inputs);
                        break;
                    case ActionType.Link:
                        ValidateLink(playerObject, x.Inputs);
                        break;
                    case ActionType.Unlink:
                        ValidateUnlink(playerObject, x.Inputs);
                        break;
                    case ActionType.Connect: 
                        ValidateConnect(playerObject, x.Inputs);
                        break;
                    case ActionType.Disconnect:
                        ValidateDisconnect(playerObject, x.Inputs);
                        break;
                        
                    default:
                        ValidateWait(playerObject, x.Inputs);
                        break;
                }

            }
            OnActionsValidated?.Invoke(_actions);
        }

        // legyen akadaly a pozicion
        private void ValidateClean(GameObject playerObject, Vector3[] inputs)
        {
            Vector3Int vec = Vector3Int.RoundToInt(inputs[0]);
            GameObject obj = MapManager.Instance.CheckCollision(vec.x, vec.y);
            Debug.Log(obj == null);
            if (obj != null && obj.CompareTag("Obstacle"))
            {
                RobotAction action = new CleanRobotAction(obj);
                _actions.Add(action);
            }
        }

        // mindket pozicion van kocka
        private void ValidateLink(GameObject playerObject, Vector3[] input)
        {
            Vector3 vec1 = Vector3Int.RoundToInt(input[0]);
            Vector3 vec2 = Vector3Int.RoundToInt(input[1]);
            GameObject cube1 = null, cube2 = null;

            foreach (GameObject o in MapManager.Instance.mapObjects)
            {
                if (o.CompareTag("Cube"))
                {
                    if ((int)o.transform.position.x == (int)vec1.x && (int)o.transform.position.y == (int)vec1.y)
                    {
                        cube1 = o;
                    }
                    else if ((int)o.transform.position.x == (int)vec2.x && (int)o.transform.position.y == (int)vec2.y)
                    {
                        cube2 = o;
                    }
                }
            }

            if (cube1 != null && cube2 != null)
            {
                RobotAction action = new LinkRobotAction(cube1, cube2);
                _actions.Add(action);
            }
        }

        // a movebject alatti osszes object tud oda forogni
        // (player szerint kell validalni)
        private void ValidateRotate(GameObject playerObject, Vector3[] input)
        {
            Vector3Int vec = Vector3Int.RoundToInt(input[0]);
            Vector3Int pos = Vector3Int.RoundToInt(playerObject.transform.position);
            int direction = vec.x - pos.x;
            RobotAction action = new RotateRobotAction(playerObject, direction);
            _actions.Add(action);
        }
        
        // mindket pozicion van kocka, amik ugyana a moveobject alatt vannak es szet tudnak esni ket kulonallo karavanra
        private void ValidateUnlink(GameObject playerObject, Vector3[] input)
        {
            Vector3 vec1 = Vector3Int.RoundToInt(input[0]);
            Vector3 vec2 = Vector3Int.RoundToInt(input[1]);
            GameObject cube1 = null, cube2 = null;

            foreach (GameObject o in MapManager.Instance.mapObjects)
            {
                if (o.CompareTag("Cube"))
                {
                    if ((int)o.transform.position.x == (int)vec1.x && (int)o.transform.position.y == (int)vec1.y)
                    {
                        cube1 = o;
                    }
                    else if ((int)o.transform.position.x == (int)vec2.x && (int)o.transform.position.y == (int)vec2.y)
                    {
                        cube2 = o;
                    }
                }
            }

            if (cube1 != null && cube2 != null)
            {
                RobotAction action = new UnlinkRobotAction(cube1, cube2);
                _actions.Add(action);
            }
        }

        // minden object a moveobject alatt tud oda mozogni ahova akar
        private void ValidateMove(GameObject playerObject, Vector3[] input)
        {
            Vector3 vec = Vector3Int.RoundToInt(input[0]) - playerObject.transform.position;
            GameObject mo = playerObject.GetComponent<Movable>().moveObject;
            RobotAction robotAction = new MoveRobotAction(mo,vec);
            _actions.Add(robotAction);
        }

        // oke
        private void ValidateWait(GameObject playerObject, Vector3[] input)
        {
            RobotAction robotAction = new WaitRobotAction(playerObject);
            _actions.Add(robotAction);
            
        }
        
        // az adott pozicion van kocka
        private void ValidateConnect(GameObject playerObject, Vector3[] input)
        {
            Vector3 vec = Vector3Int.RoundToInt(input[0]);
            int x = (int)vec.x;
            int y = (int)vec.y;

            //var cubeobject= MapManager.Instance.CheckCollision(x, y);
            //cubeobject.CompareTag("Cube");
            
            foreach (GameObject o in MapManager.Instance.mapObjects)
            {
                if ((int)o.transform.position.x == x && (int)o.transform.position.y == y)
                {
                    RobotAction action = new ConnectRobotAction(playerObject, o);
                    _actions.Add(action);
                    return;
                }
            }
        }
        
        // az adott pozicion van kocka
        private void ValidateDisconnect(GameObject playerObject, Vector3[] input)
        {
            Vector3 vec = Vector3Int.RoundToInt(input[0]);
            int x = (int)vec.x;
            int y = (int)vec.y;
            
            foreach (GameObject o in MapManager.Instance.mapObjects)
            {
                if ((int)o.transform.position.x == x && (int)o.transform.position.y == y)
                {
                    RobotAction action = new DisconnectRobotAction(playerObject, o);
                    _actions.Add(action);
                    return;
                }
            }
        }
    
    }
}