﻿using System;
using System.Collections.Generic;
using Model;
using MultiPlayer.LobbyScripts;
using NetworkBehaviourClasses;
using UI.DevConsole;
using Unity.Netcode;
using Unity.Services.Authentication;
using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.SceneManagement;
using Random = UnityEngine.Random;

namespace MultiPlayer
{
    public class GameNetworkManager : NetworkBehaviour
    {
        
        
        
        private ulong _connectedCounter = 0;
        private Dictionary<ulong,KeyValuePair<string,string>> _nameToIDDictionary;
        
        public List<InputData> CurrentInputDatas { get; private set; }
        public static event Action<Dictionary<ulong,KeyValuePair<string,string>>> OnReady;

       public static GameNetworkManager Instance { get; private set; }
       
        private void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
            }
            Instance = this;
            DontDestroyOnLoad(this);
            _nameToIDDictionary = new Dictionary<ulong,KeyValuePair<string,string>>();
            CurrentInputDatas = new List<InputData>();
            GameManager.OnGameStateChanged += GameManagerOnGameStateChanged;
        }

        private void GameManagerOnGameStateChanged(GameState state)
        {
            if (state == GameState.Turn)
            {
                ClearTurnData();
            }
        }

        private void Start()
        {
            NetworkManager.Singleton.OnClientConnectedCallback += connectedPlayers =>
            {
                if(!IsHost) return;
                
                ++_connectedCounter;
                Debug.Log($"<color=orange>Player Connected NetCode ID: {connectedPlayers}, Number of Players: {_connectedCounter}</color>");
                
                if (_connectedCounter == LobbyManager.Instance.ConnectedPlayers)
                {
                    GetNameClientRpc();
                }    
            };
        }
        
        

        private void Update()
        {
        }


        private void SetUpScene()
        {
            OnReady?.Invoke(_nameToIDDictionary);
        }
        
        
        private void LoadNextScene()
        {
            Debug.Log("<color=yellow>Loading Scene</color>");
            Unity.Netcode.NetworkManager.Singleton.SceneManager.LoadScene("gameScene",LoadSceneMode.Single);
            NetworkManager.Singleton.SceneManager.OnSceneEvent += LoadFinished;

        }

        private void LoadFinished(SceneEvent sceneevent)
        {
            if (sceneevent.SceneEventType == SceneEventType.LoadEventCompleted)
            {
                SetUpScene();
            }
        }

        [ServerRpc(RequireOwnership = false)]
        private void SendNameServerRpc(string playerName, string team, ServerRpcParams serverRpcParams = default)
        {
            _nameToIDDictionary.Add(serverRpcParams.Receive.SenderClientId , new KeyValuePair<string, string>(playerName,team));
            Debug.Log($"<color=orange>Name: {playerName}, Joined Team: {team}, ID: {serverRpcParams.Receive.SenderClientId}</color>");
            if ((ulong)_nameToIDDictionary.Count == LobbyManager.Instance.ConnectedPlayers)
            {
                LoadNextScene();
            }
        }

        [ClientRpc]
        private void GetNameClientRpc()
        {
            Debug.Log("<color=yellow>Sending Name</color>");
            SendNameServerRpc(AuthenticationService.Instance.Profile,LobbyManager.Instance.joinedTeam);
        }


        public void ClearTurnData()
        {
            CurrentInputDatas.Clear();
        }

        [DevCommands("List")]
        public void ListCurrentInputData()
        {
            foreach (var data in CurrentInputDatas)
            {
                Debug.Log($"Positions: {data.Inputs[0]},{data.Inputs[1]}, Type: {data.Type}");
            }
        }
        
        [ServerRpc(RequireOwnership = false)]
        public void SendDataServerRpc(NetworkObjectReference playerObject , Vector3[] inputs, ActionType type ,ServerRpcParams serverRpcParams = default)
        {
            CurrentInputDatas.Add(new InputData(playerObject,inputs,type));
            if (CurrentInputDatas.Count == (int)_connectedCounter)
            {
                ActionValidator validator = new ActionValidator(CurrentInputDatas, GameManager.Instance);
                validator.Validate();
            }
        }
        
        
    }
    public struct InputData : IComparable<InputData>
    {
        public NetworkObjectReference PlayerObject;
        public Vector3[] Inputs;
        public ActionType Type;

        public InputData(NetworkObjectReference playerObject,Vector3[] inputs,ActionType type)
        {
            PlayerObject = playerObject;
            Inputs = inputs;
            Type = type;
        }
        public int CompareTo(InputData other)
        {
            return Type.CompareTo(other.Type);
        }
    }
}