using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading.Tasks;
using MultiPlayer.Relay;
using Unity.Netcode;
using Unity.Services.Authentication;
using Unity.Services.Lobbies;
using Unity.Services.Lobbies.Models;
using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.Serialization;
using Task = System.Threading.Tasks.Task;

namespace MultiPlayer.LobbyScripts
{
    public class LobbyManager : MonoBehaviour
    {
        public static LobbyManager Instance { get; private set; }
        public event EventHandler<LobbyEventArgs> LobbyListChangedEvent;
        public event EventHandler<LobbyEventArgs> LobbyChangedEvent;

        private Lobby _hostLobby;
        private Lobby _joinedLobby;
        private const float HeartBeatTimer = 14f;
        private float _lobbyUpdateTimer = 1.1f;
        private IEnumerator _coroutine;
        private bool _crRunning;
        private int _maxPlayers;

        public string joinedTeam;
        
        public ulong ConnectedPlayers { get; private set; }
        
            
        void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }

            Instance = this;
        }

        private void Update()
        {
            PullForLobbyUpdates();
        }
        
        /// <summary>
        /// Heart beat Coroutine
        /// </summary>
        /// <remarks>
        /// Keeps the lobby alive and findable for the lobby list
        /// </remarks>
        /// <param name="lobbyId">The lobby to ping</param>
        /// <param name="waitTimeSeconds">The time to wait before an other update</param>
        /// <returns></returns>
        IEnumerator HeartbeatLobbyCoroutine(string lobbyId, float waitTimeSeconds)
        {
            _crRunning = true;
            var delay = new WaitForSecondsRealtime(waitTimeSeconds);

            while (true)
            {

                LobbyService.Instance.SendHeartbeatPingAsync(lobbyId);
                yield return delay;
            }
        }
        
        
        /// <summary>
        /// Hosts a lobby for the game
        /// </summary>
        /// <param name="name">Name of the lobby</param>
        /// <param name="maxPlayer">Max number of players in the lobby</param>
        public async void CreateLobby(string name, int maxPlayer)
        {
            try
            {
                CreateLobbyOptions createLobbyOptions = new CreateLobbyOptions{
                    Player = NewPlayer("Blue"),
                    Data = new Dictionary<string, DataObject>
                    {
                        {"GameStart",new DataObject(DataObject.VisibilityOptions.Member,"0")}
                    }
                };
                
                _hostLobby = await LobbyService.Instance.CreateLobbyAsync(name, maxPlayer,createLobbyOptions);
                Debug.Log($"<color=green>LobbyCreationSuccessFull {_hostLobby.Name},{_hostLobby.MaxPlayers}</color>");
                _joinedLobby = _hostLobby;
                ListPlayers(_hostLobby);
                _maxPlayers = maxPlayer;
                _coroutine = HeartbeatLobbyCoroutine(_hostLobby.Id, HeartBeatTimer);
                StartCoroutine(_coroutine);
            }
            catch (Exception e)
            {
                Debug.LogWarning(e.Message);
            }
        }

        /// <summary>
        /// Lists lobbies that have already been created
        /// </summary>
        public async void ListLobbys()
        {
            try
            {
                QueryResponse queryResponse = await Lobbies.Instance.QueryLobbiesAsync();
                
                
                Debug.Log($"<color=orange>Found Lobbis:{queryResponse.Results.Count}</color>");
                foreach (var lobby in queryResponse.Results)
                {
                    Debug.Log($"<color=orange>Name:{lobby.Name}, {lobby.MaxPlayers-lobby.AvailableSlots}/{lobby.MaxPlayers}</color>");
                    
                }
                
                LobbyListChangedEvent?.Invoke(this,new LobbyEventArgs(queryResponse.Results));
            }
            catch (Exception e)
            {
                Debug.LogException(e);
            }
        }

        /// <summary>
        /// Used to join an existing lobby
        /// </summary>
        /// <param name="lobby">The lobby to join</param>
        /// <param name="team">The team to join to</param>
        /// <returns>The updated lobby information</returns>
        public async Task<Lobby> JoinLobby(Lobby lobby,string team)
        {
            try
            {
                JoinLobbyByIdOptions joinLobbyByIdOptions = new JoinLobbyByIdOptions
                {
                    Player = NewPlayer(team)
                };
                 _joinedLobby = await Lobbies.Instance.JoinLobbyByIdAsync(lobby.Id,joinLobbyByIdOptions);
                 Debug.Log("<color=green>Lobby Join Success Full</color>");
                 ListPlayers(_joinedLobby);
                 return _joinedLobby;
            }
            catch (Exception e)
            {
                Debug.LogException(e);
                return null;
            }
        }

        
        /// <summary>
        /// Creates a player with their own name and team
        /// <remarks>
        /// The name the one they gave when they first started the game
        /// </remarks>
        /// </summary>
        /// <param name="team">The team to join</param>
        /// <returns>The new player variable</returns>
        private Player NewPlayer(string team)
        {

            joinedTeam = team;
            return new Player
            {
                Data = new Dictionary<string, PlayerDataObject>
                {
                    {"PlayerName", new PlayerDataObject(PlayerDataObject.VisibilityOptions.Member,AuthenticationService.Instance.Profile) },
                    {"Team", new PlayerDataObject(PlayerDataObject.VisibilityOptions.Member,team)}
                }
            };
        }

        
        /// <summary>
        /// Used to pull for lobby changes unit
        /// <remarks>
        /// Unity Lobby does not support real time connection needed to update lobby status in the UI
        /// </remarks>
        /// </summary>
        private async void PullForLobbyUpdates()
        {
            if (_joinedLobby != null)
            {
                _lobbyUpdateTimer -= Time.deltaTime;
                if (_lobbyUpdateTimer < 0f)
                {
                    try
                    {
                        _lobbyUpdateTimer = 1.2f;
                        Lobby njoinedLobby = await LobbyService.Instance.GetLobbyAsync(_joinedLobby.Id);
                        ConnectedPlayers = Convert.ToUInt64(njoinedLobby.Players.Count);
                        LobbyChangedEvent?.Invoke(this, new LobbyEventArgs(njoinedLobby));
                        _joinedLobby = njoinedLobby;
                        if (njoinedLobby.HostId == AuthenticationService.Instance.PlayerId && !_crRunning)
                        {
                            _hostLobby = njoinedLobby;
                            _maxPlayers = njoinedLobby.MaxPlayers;
                            _coroutine = HeartbeatLobbyCoroutine(njoinedLobby.Id, HeartBeatTimer);
                            StartCoroutine(_coroutine);
                        }

                        if(njoinedLobby.Data["GameStart"].Value != "0")
                        {
                            
                            if (_hostLobby == null)
                            {
                                Debug.Log($"<color=yellow>Joining Relay as Client with code {njoinedLobby.Data["GameStart"].Value}</color>");
                                RelayManager.Instance.JoinRelay(njoinedLobby.Data["GameStart"].Value);
                            }
                            _joinedLobby = null;
                        }
                        
                        
                    }
                    catch (Exception e)
                    {
                        Debug.LogException(e);
                    }
                }
            }
        }
        
        
        /// <summary>
        /// Lists the members of the lobby for debug purposes only 
        /// </summary>
        /// <param name="lobby">The lobby it lists from</param>
        private void ListPlayers(Lobby lobby)
        {
            foreach (Player player in lobby.Players)
            {
                Debug.Log($"<color=teal>Name:{player.Data["PlayerName"].Value} Team:{player.Data["Team"].Value}</color>");
            }
        }

        /// <summary>
        /// Used to Leave a Lobby
        /// </summary>
        public async void LeaveLobby()
        {
            try
            {
                if (_hostLobby != null)
                {
                    StopCoroutine(_coroutine);
                    _crRunning = false;
                    MigrateHost();
                }
                
                await LobbyService.Instance.RemovePlayerAsync(_joinedLobby.Id, AuthenticationService.Instance.PlayerId);
                _joinedLobby = null;
                _hostLobby = null;
                joinedTeam = null;
            }
            catch (Exception e)
            {
                Debug.LogException(e);
            }
        }
        
        /// <summary>
        /// Used to migrate host to the the first player in the lobby and stops the coroutine
        /// </summary>
        private async void MigrateHost()
        {
            try
            {
                if (_joinedLobby.Players.Count > 1)
                {
                    _hostLobby = await Lobbies.Instance.UpdateLobbyAsync(_hostLobby.Id, new UpdateLobbyOptions
                    {
                        HostId = _joinedLobby.Players[1].Id
                    });
                    _joinedLobby = _hostLobby;
                }
            }
            catch (Exception e)
            {
                Debug.LogException(e);
            }
        }

        public async void StartGame()
        {
            if (_hostLobby != null)
            {
                try
                {
                    _crRunning = false;
                    StopCoroutine(_coroutine);
                    string joinCode = await RelayManager.Instance.CreateRelay(_maxPlayers);
                    Lobby lobby = await Lobbies.Instance.UpdateLobbyAsync(_joinedLobby.Id, new UpdateLobbyOptions
                    {
                        Data = new Dictionary<string, DataObject>
                        {
                            {"GameStart",new DataObject(DataObject.VisibilityOptions.Member,joinCode)}
                        }
                    });
                    Debug.Log("<color=yellow>Starting Game</color>");
                    _joinedLobby = lobby;

                }
                catch (Exception e)
                {
                    Debug.LogException(e);
                }
            }
        }

    }
}
