﻿using System;
using System.Collections.Generic;
using System.Linq;
using Model;
using MultiPlayer;
using Unity.Netcode;
using Unity.VisualScripting;
using UnityEditor;
using UnityEngine;
using UnityEngine.Serialization;
using GameObject = UnityEngine.GameObject;
using Random = UnityEngine.Random;

namespace NetworkBehaviourClasses
{
    public class MapManager : NetworkBehaviour
    {
        
        [SerializeField] private GameObject cube;
        [SerializeField] private GameObject moveObject;
        [SerializeField] private GameObject obstacle;
        [SerializeField] private GameObject edge;
        [SerializeField] private GameObject player;
        
        public List<GameObject> mapObjects = new List<GameObject>();
        public static MapManager Instance { get; private set; } // singleton
        public int mapSize;
        
        private void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }

            Instance = this;
            GameNetworkManager.OnReady += GameNetworkManagerOnReady;
        }

        private void GameNetworkManagerOnReady(Dictionary<ulong, KeyValuePair<string, string>> dict)
        {
            if(!IsHost) return;
            mapSize = 2 + dict.Count / 2;
            InitMap();
            InitSpawnCubes();
            InitSpawnPlayers(dict);
        }

        public override void OnNetworkSpawn()
        {
            if(!IsHost) return;
            base.OnNetworkSpawn();
        }
        
        public GameObject CreateMoveObject()
        {
            GameObject obj = Instantiate(moveObject, new Vector3(3, 2, 0), Quaternion.identity);
            obj.GetComponent<NetworkObject>().Spawn();
            return obj;
        }

        public void KillMapObject(GameObject obj)
        {
            mapObjects.Remove(obj);
            obj.GetComponent<NetworkObject>().Despawn();
        }

        public GameObject CheckCollision(int x, int y)
        {
            foreach (GameObject obj in mapObjects)
            {
                Vector3Int pos = Vector3Int.RoundToInt(obj.transform.position);
                if (pos.x == x && pos.y == y) return obj;
            }
            return null;
        }

        private void InitSpawnCubes()
        {
            foreach (CubeColor color in Enum.GetValues(typeof(CubeColor)).Cast<CubeColor>())
            {
                int n = Random.Range(mapSize, mapSize * 2 + 1);
                for (int i = 0; i < n; i++)
                {
                    SpawnCube(color);
                }
            }
        }

        private void SpawnCube(CubeColor color)
        {
            int x, y;
            do
            {
                x = Random.Range(-(mapSize * 10 / 2) + 1, mapSize * 10 / 2 - 1) ;
                y = Random.Range(-(mapSize * 10 / 2) + 1, mapSize * 10 / 2 - 1);
            } while (CheckCollision(x, y));
            
            GameObject cubeObject = Instantiate(cube, new Vector3(x, y, 0), Quaternion.identity);
            cubeObject.GetComponent<NetworkObject>().Spawn();
            cubeObject.GetComponent<Cube>().SetColorClientRpc(color);
            mapObjects.Add(cubeObject);
        }
        
        private void InitSpawnPlayers(Dictionary<ulong, KeyValuePair<string, string>> dict)
        {
            foreach (var keyValuePair in dict)
            {
                int x, y;
                do
                {
                    x = Random.Range(-(mapSize * 10 / 2) + 1, mapSize * 10 / 2 - 1) ;
                    y = Random.Range(-(mapSize * 10 / 2) + 1, mapSize * 10 / 2 - 1);
                } while (CheckCollision(x, y));
                
                GameObject playerObject = Instantiate(player, new Vector3(x,y,0), Quaternion.identity);
                playerObject.SetActive(true);
                playerObject.GetComponent<NetworkObject>().SpawnAsPlayerObject(keyValuePair.Key,true);
                playerObject.GetComponent<Player>().SetTeamClientRpc(keyValuePair.Value.Value == "Red" ? Team.Red : Team.Blue);
                mapObjects.Add(playerObject);
            }
        }
        
        private void SpawnMapObject(int type, int x, int y)
        {
            GameObject obj;
            switch (type)
            {
                case 1:
                    obj = Instantiate(obstacle, new Vector3(x,y,0), Quaternion.identity);
                    obj.GetComponent<NetworkObject>().Spawn();
                    mapObjects.Add(obj);
                    if (Random.value < 0.2) obj.GetComponent<Obstacle>().Hit();
                    break;
                case 2:
                    obj = Instantiate(edge, new Vector3(x, y, 0), Quaternion.identity);
                    obj.GetComponent<NetworkObject>().Spawn();
                    mapObjects.Add(obj);
                    break;
            }
        }

        private void InitMap()
        {
            MapSegmentBank bank = new MapSegmentBank();
            int nCorners = 4;
            int nSides = 4 * (mapSize - 2);
            int nInternals = (mapSize - 2) * (mapSize - 2);
            int mapHalf = 10 * mapSize / 2;

            Func<int, int, (int, int)> cornerTransform = (x, y) => (x,y);
            Func<int, int, int, (int, int)> sideTransform = (x, y, k) => (x,y);
            Func<int, int, int, (int, int)> internalTransform = (x, y, k) => (-mapHalf + 10 + 10 * (k % (mapSize - 2)) + x, mapHalf - 10 - 10 * (k / (mapSize - 2)) - y);

            for (int n = 0; n < nCorners; n++)
            {
                switch (n)
                {
                    case 0:
                        cornerTransform = (x, y) => (-mapHalf + x, mapHalf - y - 1);
                        sideTransform = (x, y, k) => (-mapHalf + 10 + k * 10 + y, mapHalf - x - 1);
                        break;
                    case 1:
                        cornerTransform = (x, y) => (mapHalf - x - 1, mapHalf - y - 1);
                        sideTransform = (x, y, k) => (mapHalf - x - 1, mapHalf - 10 - 10 * k - y - 1);
                        break;
                    case 2:
                        cornerTransform = (x, y) => (mapHalf - x - 1, -mapHalf + y);
                        sideTransform = (x, y, k) => (-mapHalf + 10 + 10 * k + (9 - y), -mapHalf + x);
                        break;
                    case 3:
                        cornerTransform= (x, y) => (-mapHalf + x, -mapHalf + y);
                        sideTransform = (x, y, k) => (-mapHalf + x, mapHalf - 10 - 10 * k - y - 1);
                        break;
                }

                int[,] cornerPattern = bank.corners[Random.Range(0, bank.corners.Count)];
                for (int i = 0; i < 10; i++)
                {
                    for (int j = 0; j < 10; j++)
                    {
                        (int cornerX, int cornerY) = cornerTransform(j, i);
                        SpawnMapObject(cornerPattern[i, j], cornerX, cornerY);
                    }
                }

                for (int k = 0; k < nSides / 4; k++)
                {
                    int[,] sidePattern = bank.sides[Random.Range(0, bank.sides.Count)];
                    for (int i = 0; i < 10; i++)
                    {
                        for (int j = 0; j < 10; j++)
                        {
                            (int sideX, int sideY) = sideTransform(j, i, k);
                            SpawnMapObject(sidePattern[i,j], sideX, sideY);
                        }
                    }
                }

            }
            for (int k = 0; k < nInternals; k++)
            {
                int[,] internalPattern = bank.internals[Random.Range(0, bank.internals.Count)];
                for (int i = 0; i < 10; i++)
                {
                    for (int j = 0; j < 10; j++)
                    {
                        (int x, int y) = internalTransform(j,i, k);
                        SpawnMapObject(internalPattern[i,j], x,y);
                    }
                }
            }
            
            InitOpenGates();
        }

        private void InitOpenGates()
        {
            int mapHalf = mapSize * 10 / 2;
            int gateSize = TaskManager.PatternGridSize - 1;
            
            for (int i = 0; i < 4; i++)
            {
                int nGates = Random.Range(2, mapSize + 1);
                List<int> gates = new List<int>();

                for (int j = 0; j < nGates; j++)
                {
                    int gate;
                    bool collision;
                    
                    do
                    {
                        collision = false;
                        gate = Random.Range(-mapHalf + 1, mapHalf - 1 - gateSize);
                        foreach (int g in gates)
                        {
                            if (g - gateSize < gate && g + gateSize > gate) collision = true;
                        }
                    } while (collision);
                    
                    gates.Add(gate);
                    OpenGate(i, gate, gateSize);
                }
            }
        }

        private void OpenGate(int side, int pos, int gateSize)
        {
            int x = 0, y = 0;
            int mapHalf = mapSize * 10 / 2;
            bool verticalGate = true;
            
            switch (side)
            {
                case 0:
                    x = pos;
                    y = mapHalf - 1;
                    break;
                case 1:
                    x = mapHalf - 1;
                    y = pos;
                    verticalGate = false;
                    break;
                case 2:
                    x = pos;
                    y = -mapHalf;
                    break;
                case 3:
                    x = -mapHalf;
                    y = pos;
                    verticalGate = false;
                    break;
            }

            for (int i = 0; i < gateSize; i++)
            {
                GameObject obstacleObject = CheckCollision(x, y);
                if (obstacleObject)
                {KillMapObject(obstacleObject);}
                else {Debug.Log("ERROR" + x + ", " + y);}

                if (verticalGate)
                {
                    x++;
                }
                else
                {
                    y++;
                }
            }
        }

        public void ScatterMoveObject(GameObject moveGameObject)
        {
            while (moveGameObject.transform.childCount > 0)
            {
                GameObject child = moveGameObject.transform.GetChild(0).gameObject;
                child.GetComponent<Movable>().Disconnect();
                Movement movement = child.GetComponent<Movement>();
                
                int x, y;
                do
                {
                    x = Random.Range(-(mapSize * 10 / 2) + 1, mapSize * 10 / 2 - 1) ;
                    y = Random.Range(-(mapSize * 10 / 2) + 1, mapSize * 10 / 2 - 1);
                } while (CheckCollision(x, y));

                Vector3 newPos = new Vector3(x, y, 0);
                
                if (movement != null) movement.movePoint = newPos;
                child.transform.position = newPos;
            }
        }
    }
}