using System;
using System.Diagnostics;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Model;
using Scriptable_Objects;
using Unity.Netcode;
using Unity.VisualScripting.Antlr3.Runtime.Tree;
using UnityEngine;
using UnityEngine.Rendering.Universal;
using UnityEngine.Serialization;
using Debug = UnityEngine.Debug;
using Random = UnityEngine.Random;

namespace NetworkBehaviourClasses
{
    public class TaskManager : NetworkBehaviour
    {
        // serialized fields
        [SerializeField] private int minCubes = 3;
        [SerializeField] private int maxCubes = 6;
        
        // private fields
        public TaskData currentTask;
        private TaskPatternBank _bank;
        
        // public fields
        public static TaskManager Instance { get; private set; } // singleton
        public static event Action<TaskData> OnTaskChanged;
        public const int PatternGridSize = 5;

        // private methods
        private void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }

            Instance = this;
        }
            
        private void Start()
        {
            if (!IsHost) return;
            _bank = new TaskPatternBank();
            currentTask = GenerateTask();
            GameManager.OnGameStateChanged += GameManagerOnOnGameStateChanged; 
        }
        
        private void GameManagerOnOnGameStateChanged(GameState state)
        {
            if (state == GameState.Turn)
            {
                currentTask.turns--;
                if (currentTask.turns == 0)
                {
                    currentTask = GenerateTask();
                }
            }
        }

        public void CreateNewTask()
        {
            currentTask = GenerateTask();
        }
    
        private TaskData GenerateTask()
        {
            TaskData task = ScriptableObject.CreateInstance<TaskData>();
            
            int n = Random.Range(minCubes, maxCubes + 1);
            (task.Pattern, n) = GeneratePattern();
            task.points = n + Random.Range(0, 3);
            task.turns = 5 * n + Random.Range(-5, 6) + 50;
            OnTaskChanged?.Invoke(task);
            return task;
        }

        public bool ValidatePattern(GameObject o)
        {
            int offsetX = Int32.MaxValue;
            int offsetY = Int32.MinValue;
            
            // find offset values
            for (int i = 0; i < o.transform.childCount; i++)
            {
                GameObject child = o.transform.GetChild(i).gameObject;
                Vector3 pos = child.transform.position;
                int x = (int)pos.x;
                int y = (int)pos.y;
                
                if (x < offsetX)
                {
                    offsetX = x;
                }
                if (y > offsetY)
                {
                    offsetY = y;
                }
            }
            
            // create pattern from the input
            int[,] pattern = new int[PatternGridSize, PatternGridSize];
            for (int i = 0; i < PatternGridSize; i++)
            {
                for (int j = 0; j < PatternGridSize; j++)
                {
                    pattern[i, j] = -1;
                }
            }

            for (int i = 0; i < o.transform.childCount; i++)
            {
                GameObject child = o.transform.GetChild(i).gameObject;
                Vector3 pos = child.transform.position;
                
                int x = (int)pos.x - offsetX;
                int y = -(int)pos.y + offsetY;
                if (!(x < PatternGridSize && y < PatternGridSize)) return false;
                
                if (child.CompareTag("Cube"))
                {
                    Cube c = child.GetComponent<Cube>();
                    pattern[y, x] = (int)c.color;
                }
                else
                {
                    pattern[y, x] = 4;
                }
            }
            
            // validate the pattern against the current task's pattern
            for (int i = 0; i < PatternGridSize; i++)
            {
                for (int j = 0; j < PatternGridSize; j++)
                {
                    if (pattern[i, j] != currentTask.Pattern[i, j])
                    {
                        return false;
                    }
                }
            }
            
            return true;
        }

        private (int[,], int) GeneratePattern()
        {
            int[,] pattern = _bank.patterns[Random.Range(0, _bank.patterns.Count)];
            int n = 0;
            int[,] newPattern = new int[PatternGridSize, PatternGridSize];
            
            Func<int, int, (int, int)> transformationLambda = (x, y) => (x, y);
            int transformationType = Random.Range(0, 4);
            
            int[] usedcolors = new int[Cube.NColors];
            for (int i = 0; i < Cube.NColors; i++) {usedcolors[i] = 0;}

            switch (transformationType)
            {
                case 0:
                    break;
                case 1:
                    transformationLambda = (x, y) => (y, x);
                    break;
                case 2:
                    transformationLambda = (x, y) => (x, PatternGridSize - y - 1);
                    break;
                case 3:
                    transformationLambda = (x, y) => (PatternGridSize - y - 1, x);
                    break;
            }
            
            for (int i = 0; i < PatternGridSize; i++)
            {
                for (int j = 0; j < PatternGridSize; j++)
                {
                    if (pattern[i, j] == 0)
                    {
                        n++;
                        int color;
                        do
                        {
                            color = Random.Range(0, 4);
                        } while (usedcolors[color] >= 2);

                        pattern[i, j] = color;
                        usedcolors[color]++;
                    }

                    (int newX, int newY) = transformationLambda(i, j);
                    newPattern[newX, newY] = pattern[i, j];
                }
            }

            newPattern = NormalizePattern(newPattern);
            
            return (newPattern, n);
        }

        private int[,] NormalizePattern(int[,] pattern)
        {
            int[,] normalizedPattern = new int[PatternGridSize, PatternGridSize];
            int offsetI = Int32.MaxValue;
            int offsetJ = Int32.MaxValue;
            
            // find offset values
            for (int i = 0; i < PatternGridSize; i++)
            {
                for (int j = 0; j < PatternGridSize; j++)
                {
                    normalizedPattern[i, j] = -1;
                    
                    if (pattern[i, j] > -1)
                    {
                        if (i < offsetI) offsetI = i;
                        if (j < offsetJ) offsetJ = j;
                    }
                }
            }
            
            for (int i = 0; i < PatternGridSize; i++)
            {
                for (int j = 0; j < PatternGridSize; j++)
                {
                    if (pattern[i, j] > -1)
                    {
                        normalizedPattern[i - offsetI, j - offsetJ] = pattern[i,j];
                    }
                }
            }

            return normalizedPattern;
        }

    }
}
