using System.ComponentModel.Design.Serialization;
using System.Threading.Tasks;
using NetworkBehaviourClasses;
using Scriptable_Objects;
using TMPro;
using Unity.Netcode;
using UnityEngine;
using UnityEngine.UI;

namespace UI
{
    public class TaskUI : NetworkBehaviour
    {
        // serialized fields
        [SerializeField] private TextMeshProUGUI infoText;
        [SerializeField] private Image gridElementPrefab;
        [SerializeField] private Sprite[] images;
        [SerializeField] private Image grid; 
        
        // private fields
        private TaskData _currentTask;

        // private methods
        private void Awake()
        {
            TaskManager.OnTaskChanged += TaskManagerOnTaskChanged;
            GameManager.OnGameStateChanged += GameManagerOnOnGameStateChanged;
        }

        private void GameManagerOnOnGameStateChanged(GameState state)
        {
            if (state == GameState.Turn)
            {
                UpdateTextClientRpc();
            }
        }

        private void TaskManagerOnTaskChanged(TaskData task)
        {
            UpdateUIClientRpc(task.points, task.turns, SerializeMatrix(task.Pattern));
        }

        private int[] SerializeMatrix(int[,] matrix)
        {
            int n = TaskManager.PatternGridSize;
            int[] arr = new int[n * n];
            int k = 0;
            for (int i = 0; i < n; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    arr[k] = matrix[i, j];
                    k++;
                }
            }

            return arr;
        }

        private int[,] DeserializeMatrix(int[] arr)
        {
            int n = TaskManager.PatternGridSize;
            int[,] matrix = new int[n, n];
            int k = 0;
            for (int i = 0; i < n; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    matrix[i, j] = arr[k];
                    k++;
                }
            }

            return matrix;
        }

        [ClientRpc]
        private void UpdateUIClientRpc(int points, int turns, int[] pattern)
        {
            _currentTask = ScriptableObject.CreateInstance<TaskData>();
            _currentTask.points = points;
            _currentTask.turns = turns;
            _currentTask.Pattern = DeserializeMatrix(pattern);
            
            UpdateText();
            GenerateImage();
        }

        [ClientRpc]
        private void UpdateTextClientRpc()
        {
            _currentTask.turns--;
            UpdateText();
        }
        
        private void UpdateText()
        {
            infoText.text = "Points: " + _currentTask.points + "\nRemaining turns: " + _currentTask.turns;
        }

        private void GenerateImage()
        {
            // delete current image
            while (grid.transform.childCount > 0) {
                DestroyImmediate(grid.transform.GetChild(0).gameObject);
            } 
            
            int[,] pattern = _currentTask.Pattern;
            int n = pattern.GetLength(0);
            
            bool[] rowsNotEmpty = { false, false, false, false, false };
            bool[] colsNotEmpty = { false, false, false, false, false };

            for (int i = 0; i < n; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    if (pattern[i, j] != -1)
                    {
                        rowsNotEmpty[i] = true;
                        colsNotEmpty[j] = true;
                    }
                }
            }

            int nCol = 0;

            foreach (var b in colsNotEmpty) { if (b) nCol++; }
            
            GridLayoutGroup layout = grid.GetComponent<GridLayoutGroup>();
            layout.constraint = GridLayoutGroup.Constraint.FixedColumnCount;
            layout.constraintCount = nCol;

            for (int i = 0; i < n; i++)
            {
                if (rowsNotEmpty[i]) {
                    for (int j = 0; j < n; j++)
                    {
                        if (colsNotEmpty[j])
                        {
                            Image element = Instantiate(gridElementPrefab, new Vector3(0, 0, 0), Quaternion.identity);
                            element.transform.SetParent(grid.transform);
                            element.transform.localScale = Vector3.one;
                            
                            if (_currentTask.Pattern[i, j] != -1)
                            {
                                element.enabled = true;
                                element.sprite = images[_currentTask.Pattern[i, j]];
                            }
                        }
                    }
                }
            }
        }
    }
}
